@@ -44,6 +44,8 @@ class Speakeasy(object):
         self.emu = None
         self.api_hooks = []
         self.code_hooks = []
+        self.dyn_code_hooks = []
+        self.invalid_insn_hooks = []
         self.mem_read_hooks = []
         self.argv = argv
         self.exit_event = exit_event
@@ -127,6 +129,14 @@ class Speakeasy(object):
             h = self.code_hooks.pop(0)
             cb, begin, end, ctx = h
             self.add_code_hook(cb, begin, end, ctx)
+        while self.dyn_code_hooks:
+            h = self.dyn_code_hooks.pop(0)
+            cb, ctx = h
+            self.add_dyn_code_hook(cb, ctx)
+        while self.invalid_insn_hooks:
+            h = self.invalid_insn_hooks.pop(0)
+            cb, ctx = h
+            self.add_invalid_instruction_hook(cb, ctx)
         while self.mem_read_hooks:
             h = self.mem_read_hooks.pop(0)
             cb, begin, end = h
@@ -346,6 +356,21 @@ class Speakeasy(object):
             return
         return self.emu.add_code_hook(cb, begin=begin, end=end, ctx=ctx, emu=self)
 
+    def add_dyn_code_hook(self, cb: Callable, ctx={}):
+        """
+        Set a callback to fire when dynamically generated/copied code is executed
+
+        args:
+            cb: Callable python function to execute
+            ctx: Optional context to pass back and forth between the hook function
+        return:
+            Hook object for newly registered hooks
+        """
+        if not self.emu:
+            self.dyn_code_hooks.append((cb, ctx))
+            return
+        return self.emu.add_dyn_code_hook(cb, ctx=ctx, emu=self)
+
     def add_mem_read_hook(self, cb: Callable, begin=1, end=0):
         """
         Set a callback to fire when a memory address is read from
@@ -410,6 +435,21 @@ class Speakeasy(object):
             return
         return self.emu.add_instruction_hook(cb, begin=begin, end=end, emu=self, insn=700)
 
+    def add_invalid_instruction_hook(self, cb: Callable, ctx=[]):
+        """
+        Set a callback to fire when an invalid instruction is attempted
+        to be executed
+
+        args:
+            cb: Callable python function to execute
+        return:
+            Hook object for newly registered hooks
+        """
+        if not self.emu:
+            self.invalid_insn_hooks.append((cb, ctx))
+            return
+        return self.emu.add_invalid_instruction_hook(cb, ctx)
+
     def add_mem_invalid_hook(self, cb: Callable):
         """
         Get a callback for when a memory access violation occurs
