@@ -32,6 +32,8 @@ GDT_FLAGS.Ring0 = 0
 IMPORT_HOOK_ADDR = 0xFEEDFACE
 DEFAULT_LOAD_ADDR = 0x40000
 
+PAGE_SIZE = 0x1000
+
 EMU_RESERVED = 0xfeedf000
 EMU_RESERVE_SIZE = 0x4000
 DYM_IMP_RESERVE = EMU_RESERVED + 0x1000
@@ -42,7 +44,8 @@ EMU_RESERVED_END = (EMU_RESERVED + EMU_RESERVE_SIZE)
 EMU_RETURN_ADDR = EMU_RESERVED
 EXIT_RETURN_ADDR = EMU_RETURN_ADDR + 1
 SEH_RETURN_ADDR = EMU_RETURN_ADDR + 4
-IMPORT_HOOK_ADDR = EMU_RETURN_ADDR + 8
+API_CALLBACK_HANDLER_ADDR = EMU_RETURN_ADDR + 8
+IMPORT_HOOK_ADDR = EMU_RETURN_ADDR + 12
 
 # Common blank DOS header
 DOS_HEADER = b'MZ\x90\x00\x03\x00\x00\x00\x04\x00\x00\x00\xff\xff\x00\x00\xb8\x00\x00\x00' \
@@ -73,6 +76,23 @@ EMPTY_PE_64 = DOS_HEADER + b'PE\x00\x00d\x86\x00\x00ABCD\x00\x00\x00\x00\x00\x00
                            b'\x00\x00\x00\x10' + (b'\x00' * 131)
 
 
+def normalize_dll_name(name):
+    ret = name
+
+    # Funnel CRTs into a single handler
+    if name.lower().startswith(('api-ms-win-crt', 'vcruntime', 'ucrtbased', 'ucrtbase', 'msvcr', 'msvcp')):
+        ret = 'msvcrt'
+
+    # Redirect windows sockets 1.0 to windows sockets 2.0
+    elif name.lower().startswith(('winsock', 'wsock32')):
+        ret = 'ws2_32'
+
+    elif name.lower().startswith('api-ms-win-core'):
+        ret = 'kernel32'
+
+    return ret
+
+
 class PeParseException(Exception):
     pass
 
@@ -118,6 +138,24 @@ class PeFile(pefile.PE):
 
         self._patch_imports()
 
+    def get_tls_callbacks(self):
+        """
+        Get the TLS callbacks for a PE (if any)
+        """
+        max_tls_callbacks = 100
+        callbacks = []
+        if hasattr(self, 'DIRECTORY_ENTRY_TLS'):
+            rva = (self.DIRECTORY_ENTRY_TLS.struct.AddressOfCallBacks -
+                   self.OPTIONAL_HEADER.ImageBase)
+
+            for i in range(max_tls_callbacks):
+                ptr = self.get_data(rva + self.ptr_size * i, self.ptr_size)
+                ptr = int.from_bytes(ptr, 'little')
+                if ptr == 0:
+                    break
+                callbacks.append(ptr)
+        return callbacks
+
     def get_emu_path(self):
         """
         Get the path of the module (as it appears to the emulated binary)
@@ -300,9 +338,10 @@ class DecoyModule(PeFile):
     We use decoy modules so that shellcode
     (or other modules) can parse the PE file to resolve exports.
     """
-    def __init__(self, path=None, data=None, fast_load=True, base=0, emu_path=''):
+    def __init__(self, path=None, data=None, fast_load=True, base=0, emu_path='', is_jitted=False):
         self.image_size = 0
         self.ep = 0
+        self.is_jitted = is_jitted
         if path or data:
             super(DecoyModule, self).__init__(path=path, data=data, fast_load=fast_load)
 
@@ -317,7 +356,7 @@ class DecoyModule(PeFile):
 
     def get_memory_mapped_image(self, max_virtual_address=0x10000000, base=None):
         mmi = super(DecoyModule, self).get_memory_mapped_image(max_virtual_address, base)
-        if len(mmi) < len(self.__data__):
+        if self.is_jitted and len(mmi) < len(self.__data__):
             return self.__data__
         return mmi
 
@@ -397,7 +436,8 @@ class JitPeFile(object):
         '''
         Update the size of the image within the optional header
         '''
-        self.basepe.OPTIONAL_HEADER.SizeOfImage = len(self.basepe.get_memory_mapped_image())
+        self.basepe.OPTIONAL_HEADER.SizeOfImage = (len(self.basepe.get_memory_mapped_image()) +
+                                                   PAGE_SIZE)
 
     def add_section(self, name, chars=0x40000040):
         '''
